--- functions

function clamp(value, min, max)
    return math.min(max, math.max(value, min))
end

function draw3DLine(init, dir, length, col, offset, force, velocity, vel)
    local Sinit = scene.camera:get(craft.camera):worldToScreen(init)
    local final = scene.camera:get(craft.camera):worldToScreen(init + (dir * length))
    local velFinal = scene.camera:get(craft.camera):worldToScreen(init + (vel / 50))
    pushStyle()
    stroke(col)
    strokeWidth(1)
    line(Sinit.x, Sinit.y, final.x, final.y)
    fill(col)
    fontSize(8)
    --[[
    text("len: " .. math.floor(length * 100) / 100, Sinit.x, Sinit.y + 2)
    text("v: " .. math.floor(velocity * 10) / 10, Sinit.x + 30, Sinit.y - 2)
    text("off: " .. math.floor(offset * 100) / 100, final.x, final.y - 10)
    text("f: " .. math.floor(force * 100) / 100, final.x + 30, final.y)
    ]]
    popStyle()
    pushStyle()
    stroke(0, 129, 255)
    strokeWidth(2)
    line(Sinit.x, Sinit.y, velFinal.x, velFinal.y)
    popStyle()
end

chunkSpawn = class()
function chunkSpawn:init()
    self.x = 0
    self.z = 0
end
function chunkSpawn:draw()
    if player.player.position.x > (self.x) then
        self.x = player.player.position.x + 200
        createGround(vec3(self.x, 0, self.z))
    end
    if player.player.position.z > (self.z) then
        self.z = player.player.position.z + 200
        createGround(vec3(self.x, 0, self.z))
    end
end

function chunkDelete()
    
    
end

playerController = class()

function playerController:init(entity, camera, x, y, z, speed, sensitivity)
    
    self.player = entity
    
    self.camera = camera
    
    self.player.x = x
    self.player.y = y
    self.player.z = z
    self.sensitivity = sensitivity
    
    self.rb = self.player:add(craft.rigidbody, DYNAMIC, 3)
    self.rb.sleepingAllowed = false
    --self.rb.linearDamping = 0.99
    
    playerBody = self.rb
    self.player:add(craft.shape.capsule, 0.5, 5)
    
    self.player.model = craft.model.cube(vec3(1,3,1), vec3(0,-1.5,0))
    self.player.material = craft.material(asset.builtin.Materials.Standard)
    self.camera.entity.parent = self.player
    self.camera.entity.position = vec3(0,0,0)
    self.camera.nearPlane = 0.1
    
    
    self.player.sleepingAllowed = false
    self.speed = speed
    self.touchId = nil
    self.touchInside = false
    self.tempTouch = {x = 0, y = 0}
    self.finalPos = {x = 0, y = 0}
    
    sceneObjects[#sceneObjects+1] = self.player
end

function playerController:draw(dt)
    
    --- rotate camera 
    local camRot = quat.eulerAngles(clamp(-(self.finalPos.y + self.tempTouch.y) * self.sensitivity, -80, 80), 0, 0)
    
    self.camera.entity.rotation = camRot
    
    --- rotate player
    self.player.eulerAngles = vec3(0,-(self.finalPos.x + self.tempTouch.x) * self.sensitivity,0)
    
    --- lock rb rotation
    self.rb.angularFactor = vec3(0,0,0)
    
    --- create movePos
    local movePos = vec3(0,0,0)
    
    ---change movePos when touchStick is being used to joystick inputs
    if moving then
        movePos = vec3(-math.cos(-joystickAngle - 1.5), 0, math.sin(-joystickAngle - 1.5 ))
        self.rb.friction = 0.5
    else
        self.rb.friction = 3
    end
    
    --- update final move to move depending on direction player is facing
    local finalMove = (movePos.z * self.player.forward) + (movePos.x * self.player.right)
    -- apply force to rb in that direction
    self.rb:applyForce(finalMove * 20)
    
    
end

function playerController:touched(touch)
    if touch.pos.x > WIDTH/2 and touch.pos.y > 170 and touch.state == BEGAN then
        self.touchId = touch.id
        self.touchInside = true
        self.initialTouch = {x = touches[self.touchId].x, y = touches[self.touchId].y}
    end
    
    if self.touchInside and touches[self.touchId].state == ENDED then
        self.touchInside = false
        self.touchId = nil
        self.finalPos = { x = self.finalPos.x + self.tempTouch.x, y = self.finalPos.y + self.tempTouch.y } 
        self.tempTouch = { x = 0, y = 0 }
    end
    
    if self.touchInside then
        self.tempTouch = { x = touches[self.touchId].x - self.initialTouch.x , y = touches[self.touchId].y - self.initialTouch.y }
    end
    
end

itemNames = class()
function itemNames:init(entity , x)
    self.name = x
end
