joystick = class()

function joystick:init( x, y, size)
    self.size = size
    self.x = x
    self.y = y
    self.handleX = self.x
    self.handleY = self.y
    self.touchId = nil
    joystickAngle = 0
    angleInDegrees = 0
    self.touchDistance = 0
    self.joystickLimit = self.size/2
    moving = false
end

function joystick:draw(invOpen)
    angleInDegrees = math.deg(joystickAngle) + 180
    if not invOpen then
        -- joystick base
        pushStyle()
        fill(136, 104, 68, 158)
        stroke(89, 65, 44, 128)
        strokeWidth(5)
        ellipse(self.x, self.y, self.size)
        popStyle()
        --joystick handle
        pushStyle()
        fill(30, 100)
        stroke(14, 115)
        strokeWidth(5)
        ellipse( self.handleX, self.handleY, self.size/2.5)
        popStyle()
    end
end

function joystick:touched(touch)
    
    if touch.state == BEGAN then 
        self.touchDistance = math.sqrt(((touch.pos.x - self.x) ^ 2) + ((touch.pos.y - self.y) ^ 2))
        if self.touchDistance < self.joystickLimit + 50 then
            self.touchId = touch.id
            moving = true
        end
    end
    if moving then
        self.touchDistance = math.sqrt(((touches[self.touchId].pos.x - self.x) ^ 2) + ((touches[self.touchId].pos.y - self.y) ^ 2))
        joystickAngle = math.atan(self.x - touches[self.touchId].pos.x, self.y - touches[self.touchId].pos.y)
        if self.touchDistance > self.joystickLimit then
            self.handleX = self.x + (self.size / 2) * math.cos(-joystickAngle - 1.5)
            self.handleY = self.y + (self.size / 2) * math.sin(-joystickAngle - 1.5) 
        else
            self.handleX = touches[self.touchId].pos.x
            self.handleY = touches[self.touchId].pos.y
        end
        if touches[self.touchId].state == ENDED then
            self.handleX = self.x
            self.handleY = self.y
            moving = false
        end
    end
end

--- Joystick / button

button = class()

function button:init(x, y, size, method, output)
    
    self.x = x
    self.y = y
    self.size = size
    self.touchId = nil
    self.pressed = false
    self.touchDistance = 0
    self.output = output
    self.method = method
end

function button:draw(icon)
    popStyle()
    fill(0, 99)
    text(icon, self.x, self.y)
    popStyle()
    pushStyle()
    fill(136, 104, 68, 158)
    stroke(89, 65, 44, 128)
    strokeWidth(5)
    ellipse(self.x, self.y, self.size)
    popStyle()
    if self.pressed then
        if self.method == HELD then
            self.output()
            if touches[self.touchId].state == ENDED then
                self.pressed = false
            end
        end
    end
    
end

function button:touched(touch)
    if not self.pressed then
        if touch.state == BEGAN then 
            self.touchDistance = math.sqrt(((touch.pos.x - self.x) ^ 2) + ((touch.pos.y - self.y) ^ 2))
            
            if self.touchDistance < self.size - 20 then
                self.touchId = touch.id
                self.pressed = true
                
            end
        end
    end
    
    if self.pressed then
        if self.method == BEGAN then 
            self.output()
            if touches[self.touchId].state == ENDED then
                self.pressed = false
            end
        end
        if touches[self.touchId].state == ENDED then
            if self.method == ENDED then 
                self.output()
            end
            self.pressed = false
        end
        if self.method == CHANGED then 
            self.output()
            self.pressed = false
        end
    end
end

--- button / wheel

swipeWheel = class()

function swipeWheel:init(x, y, size, col, sens)
    self.x = x
    self.y = y
    self.size = size
    self.color = col
    self.gripDistance = 0
    self.gripX = 0
    self.sens = sens
end

function swipeWheel:draw()
    pushStyle()
    pushMatrix()
    translate(self.x, self.y)
    rotate(self.gripDistance)
    stroke(self.color:mix(color(1), 0.5))
    strokeWidth(8)
    fill(255, 0)
    ellipse(0, 0, self.size)
    popMatrix()
    
    pushMatrix()
    strokeWidth(2)
    rectMode(CENTER)
    fill(self.color)
    translate(self.x, self.y)
    rotate(self.gripDistance)
    rect(0, 0, self.size - 15, self.size/10)
    popMatrix()
    
    pushMatrix()
    translate(self.x, self.y)
    rotate(self.gripDistance)
    translate(0, -self.size/4)
    rect(0, 0, self.size/10, self.size/2 - 16)
    popMatrix()
    
    pushMatrix()
    translate(self.x, self.y)
    ellipse(0, 0, self.size/3)
    fill(255)
    
    popMatrix()
    popStyle()
    
    if not self.touchInside then
        self.gripDistance = self.gripDistance / 2
        if self.gripDistance < 0.05 and self.gripDistance > -0.05 then
            self.gripDistance = 0
        end
    end
    
    return self.gripDistance
end

function swipeWheel:touched(touch)
    if touch.state == BEGAN then 
        self.touchDistance = math.sqrt(((touch.pos.x - self.x) ^ 2) + ((touch.pos.y - self.y) ^ 2))
        print(self.touchDistance)
        if self.touchDistance < 50 then
            self.touchInside = true
            self.touchID = touch.id
        end
    end
    --- if touch is over, reset parameters
    if self.touchInside and  touches[self.touchID].state == ENDED then
        self.gripX = 0
        self.currentAngle = 0
        self.touchInside = false
        self.touchID = nil
    end
    --- set initial touch position
    if self.touchInside then
        if self.gripX == 0 then
            self.gripX = touches[self.touchID].pos.x
        end
        --- Distance from initial touch
        self.gripDistance = math.max(-90,math.min(90,-(touches[self.touchID].pos.x - self.gripX) * self.sens))
    end
end

pedal = class()

function pedal:init(x, y, width, height, col, sens)
    self.x = x
    self.y = y
    self.width = width
    self.height = height
    self.color = col
    self.gripDistance = 0
    self.gripY = 0
    self.sens = sens or 1
end

function pedal:draw()
    pushStyle()
    rectMode(CORNERS)
    rect(self.x, self.y, self.x + self.width, self.y + self.height - self.gripDistance)
    popStyle()
    return self.gripDistance
end

function pedal:touched(touch)
    if touch.state == BEGAN then 
        if touch.pos.x > self.x and touch.pos.y > self.y and touch.pos.x < self.x + self.width and touch.pos.y < self.y + self.height then
            self.touchInside = true
            self.touchID = touch.id
        end
    end
    --- if touch is over, reset parameters
    if self.touchInside and  touches[self.touchID].state == ENDED then
        self.gripY = 0
        self.currentAngle = 0
        self.gripDistance = 0
        self.touchInside = false
        self.touchID = nil
    end
    --- set initial touch position
    if self.touchInside then
        if self.gripY == 0 then
            self.gripY = touches[self.touchID].pos.y
        end
        --- Distance from initial touch
        self.gripDistance = math.max(0,math.min(self.height - 20,-(touches[self.touchID].pos.y - self.gripY) * self.sens))
    end
end
