weldJoint = class()

function weldJoint:init(blank, bodyB, bodyA, pos)
    self.bodyA = bodyA
    self.bodyB = bodyB
end

function weldJoint:update()
    print(self.bodyA.position)
end

suspension = class()

function suspension:init(body, rest, travel, stiff, damper)
    self.body = body
    self.restLength = rest or 0.5
    self.springTravel = travel or 0.5
    self.springStiffness = stiff or 250
    self.damperStiffness = damper or 50
    
    self.minLength = self.restLength - self.springTravel
    self.maxLength = self.restLength + self.springTravel
    
    self.springLength = self.restLength
    self.lastLength = self.springLength
    
    self.wheelRadius = 0.5
    
end

function suspension:draw(right, forward)
    self.right = right
    self.forward = forward
    
    self.rayPos = self.body.position + (self.forward) + (self.right) + (self.body.up * 0.1)
    
    self.raycast = scene.physics:raycast(self.rayPos, vec3(0,-1,0), self.maxLength + self.wheelRadius, 1, 1)
    
    if self.raycast then
        self.springLength = ((self.maxLength + self.wheelRadius) *  self.raycast.fraction) - self.wheelRadius
        
    else
        self.springLength = self.lastLength
    end
    
    self.springVelocity = (self.lastLength - self.springLength) / DeltaTime
    
    self.springOffset = self.restLength - self.lastLength
    
    self.springForce =  self.springOffset * self.springStiffness
    
    self.damperForce = self.damperStiffness * self.springVelocity
    
    self.suspensionForce = (self.springForce + self.damperForce)
    
    self.forceVector = self.suspensionForce * self.body.up
    
    if self.raycast then
        self.body:get(craft.rigidbody):applyForce(vec3(0, self.forceVector.y,0), self.forward + self.right)
    end
    
    self.lastLength = self.springLength 
    
    return self.rayPos, self.springLength, self.springOffset, self.suspensionForce, self.springVelocity
end

--- suspension / wheel force

wheelForce = class()

function wheelForce:init(parent)
    self.lastPosition = nil
end

function wheelForce:draw(parent, tire, forcePos, acc, brake, hp, grip)
    
    if self.lastPosition == nil then
        self.lastPosition = parent.position
        return
    end
    
    self.velocity = (tire.position - self.lastPosition) / DeltaTime
    
    self.velRight = self.velocity:dot(tire.right)
    
    self.velForward = self.velocity:dot(tire.forward)
    
    self.force = (-tire.right * (self.velRight * (50 * grip))) + 
    (-tire.forward * (self.velForward * (50 * brake))) + 
    (tire.forward * ((hp - 50) * acc))
    
    parent:get(craft.rigidbody):applyForce(self.force, forcePos )
    
    self.lastPosition = tire.position
    
    return -self.force
end

spring = class()

function spring:init(pos)
    self.lastPosition = pos
end

function spring:draw(entity, targetPosition, strength, damper, lastPosition)
    
    local velocity = (self.lastPosition - entity.position) / DeltaTime
    
    local offset = entity.position - targetPosition
    
    local force = offset * strength
    
    local dampening = damper * velocity
    
    local finalForce = force - dampening
    
    entity:get(craft.rigidbody):applyForce(-finalForce)
    
    self.lastPosition = entity.position
    
end

socket = class()

function socket:init(parent)
    self.socketGizmo = scene:entity()
    self.socketGizmo = scene:entity()
    self.socketGizmo.model = craft.model(asset.builtin.Primitives.Sphere)
    self.socketGizmo.material = craft.material(asset.documents.Glass)
    self.socketGizmo.material.diffuse = color(102, 255, 0, 120)
    self.socketGizmo.material.roughness = 1
    self.socketGizmo.scale = vec3(1,1,1) * 0.25
    self.socketUsed = false
end

function socket:draw(parent, group, scanPos, attachPos, attachRot)
    self.socketGizmo.position = parent.position + scanPos
    self.attachRay = scene.physics:spherecast(self.socketGizmo.position, vec3(0,1,0), 0.1, 0.25)
    
    if self.attachRay and not self.socketUsed then
        if self.attachRay.entity:get(craft.rigidbody).group == group then
            self.socketObject = self.attachRay.entity
            self.socketUsed = true
        end
    end
    if self.socketUsed then 
        
        if holdingItem and heldItem == self.socketObject then
            self.socketObject:get(craft.rigidbody).type = DYNAMIC
            self.socketObject = nil
            self.socketUsed = false
        else
            self.socketObject:get(craft.rigidbody).type = STATIC
            self.socketObject.position = attachPos
            self.socketObject.rotation = attachRot
        end
    end
end